@@ -71,6 +71,19 @@ class DatasetRecordsConsumer(threading.Thread):
         """Runs the consumer."""
         while self.running:
             self.log_next_batch()
+        self._log_remaining()
+
+    def _log_remaining(self):
+        """When the consumer is paused, send remaining data in the queue"""
+        last_batch = []
+        while not self.queue.empty():
+            last_batch.append(self.queue.get())
+        if not len(last_batch):
+            return
+
+        self._log_records(last_batch)
+        for _ in last_batch:
+            self.queue.task_done()
 
     def pause(self):
         """Pause the consumer."""
@@ -78,24 +91,16 @@ class DatasetRecordsConsumer(threading.Thread):
 
     def log_next_batch(self):
         """Upload the next batch of items, return whether successful."""
-        success = False
+
         batch = self._next_batch()
         if len(batch) == 0:
             return False
 
-        try:
-            self._log_records(batch)
-            success = True
-        except Exception as e:
-            self.log.error("error logging data: %s", e)
-            success = False
-            if self.on_error:
-                self.on_error(e, batch)
-        finally:
-            # mark items as acknowledged from queue
-            for _ in batch:
-                self.queue.task_done()
-            return success
+        success = self._log_records(batch)
+        # mark items as acknowledged from queue
+        for _ in batch:
+            self.queue.task_done()
+        return success
 
     def _next_batch(self) -> List[Record]:
         queue = self.queue
@@ -130,17 +135,23 @@ class DatasetRecordsConsumer(threading.Thread):
             max_tries=self.retries + 1,
             giveup=fatal_exception,
         )
-        def _inner_log_records():
+        def _inner_log_records(batch_: List[Record]):
             self.api.log(
                 name=self.dataset,
-                records=batch,
+                records=batch_,
                 tags=self.tags,
                 metadata=self.metadata,
-                background=True,
                 verbose=False,
             )
 
-        _inner_log_records()
+        try:
+            _inner_log_records(batch)
+            return True
+        except Exception as e:
+            self.log.error("error logging data: %s", e)
+            if self.on_error:
+                self.on_error(e, batch)
+            return False
 
     def send(self, records: Iterable[Record]):
         """Send records to the consumer"""
