@@ -22,6 +22,7 @@ from opensearchpy.exceptions import (
     OpenSearchWarning,
     RequestError,
 )
+from opensearchpy.helpers import BulkIndexError
 
 from argilla.server.daos.backend import query_helpers
 from argilla.server.daos.backend.base import BackendErrorHandler, IndexNotFoundError
@@ -53,6 +54,7 @@ class OpenSearchClient(IClientAdapter):
         self.error_handling = BackendErrorHandler(
             WarningIgnore=OpenSearchWarning,
             RequestError=RequestError,
+            BulkError=BulkIndexError,
             NotFoundError=NotFoundError,
             GenericApiError=OpenSearchException,
         )
@@ -148,7 +150,7 @@ class OpenSearchClient(IClientAdapter):
                 refresh="wait_for",
             )
 
-    def _reindex(
+    def reindex(
         self,
         *,
         source_index: str,
@@ -399,30 +401,30 @@ class OpenSearchClient(IClientAdapter):
         override: bool = True,
         reindex: bool = False,
     ):
-
-        source_index = self._get_original_index_name(source_index)
-
         if reindex:
-            return self._reindex(source_index=source_index, target_index=target_index)
-
-        is_source_read_only_index = self._is_read_only_index(index=source_index)
+            return self.reindex(
+                source_index=source_index,
+                target_index=target_index,
+            )
 
+        is_source_read_only_index = self.is_read_only_index(index=source_index)
         try:
             if not is_source_read_only_index:
-                self._enable_or_disable_read_only_index(
-                    index=source_index, read_only=True
-                )
-
+                self.enable_read_only_index(index=source_index)
             if override:
                 self.delete_index(index=target_index)
-
-            self._clone_index(index_from=source_index, index_to=target_index)
+            self._clone_index(
+                index_from=source_index,
+                index_to=target_index,
+            )
         finally:
             self._enable_or_disable_read_only_index(
-                index=source_index, read_only=is_source_read_only_index
+                index=source_index,
+                read_only=is_source_read_only_index,
             )
             self._enable_or_disable_read_only_index(
-                index=target_index, read_only=is_source_read_only_index
+                index=target_index,
+                read_only=is_source_read_only_index,
             )
 
     def _clone_index(
@@ -680,7 +682,7 @@ class OpenSearchClient(IClientAdapter):
                 wait_for_active_shards=self.index_shards,
             )
 
-    def _is_read_only_index(self, index: str) -> bool:
+    def is_read_only_index(self, index: str) -> bool:
         with self.error_handling(index=index):
             response = self.__client__.indices.get_settings(
                 index=index,
@@ -688,10 +690,23 @@ class OpenSearchClient(IClientAdapter):
                 allow_no_indices=True,
                 flat_settings=True,
             )
-            if not response:
-                return False
+            return (
+                response[index]["settings"]["index.blocks.write"] == "true"
+                if response
+                else False
+            )
 
-            return response[index]["settings"]["index.blocks.write"] == "true"
+    def enable_read_only_index(self, index: str):
+        return self._enable_or_disable_read_only_index(
+            index=index,
+            read_only=True,
+        )
+
+    def disable_read_only_index(self, index: str):
+        return self._enable_or_disable_read_only_index(
+            index=index,
+            read_only=False,
+        )
 
     def set_index_settings(
         self,
@@ -779,7 +794,7 @@ class OpenSearchClient(IClientAdapter):
                 is_phrase_query=is_phrase_query,
             )
             if keywords:
-                data["search_keywords"] = keywords
+                data[highlight.search_keywords_field] = keywords
 
         return data
 
@@ -802,9 +817,3 @@ class OpenSearchClient(IClientAdapter):
             size=size,
         )
         return results
-
-    def _get_original_index_name(self, source_index_or_alias: str):
-        response = self.__client__.indices.get(index=source_index_or_alias)
-        # The response contains a single key with the original index name (if we provided an alias)
-        for index_name in response:
-            return index_name
